program extract;
{$APPTYPE CONSOLE}
uses
  SysUtils, Classes;

// Recursive procedure to build a list of files
// from: http://www.festra.com/eng/snip04.htm
procedure FindFiles(FilesList: TStringList; StartDir, FileMask: string);
var
  SR: TSearchRec;
  DirList: TStringList;
  IsFound: Boolean;
  i: integer;
begin
  if StartDir[length(StartDir)] <> '\' then
    StartDir := StartDir + '\';

  { Build a list of the files in directory StartDir
     (not the directories!)                         }

  IsFound := FindFirst(StartDir+FileMask, faAnyFile-faDirectory, SR) = 0;
  while IsFound do begin
    FilesList.Add(StartDir + SR.Name);
    IsFound := FindNext(SR) = 0
  end;
  FindClose(SR);

  // Build a list of subdirectories
  DirList := TStringList.Create;
  IsFound := FindFirst(StartDir+'*.*', faAnyFile, SR) = 0;
  while IsFound do begin
    if ((SR.Attr and faDirectory) <> 0) and
         (SR.Name[1] <> '.') then DirList.Add(StartDir + SR.Name);
    IsFound := FindNext(SR) = 0
  end;
  FindClose(SR);

  // Scan the list of subdirectories
  for i := 0 to DirList.Count - 1 do
    FindFiles(FilesList, DirList[i], FileMask);

  DirList.Free
end;

// T_INV        // an invalid token
// T_NA         // an invalid token
// T_CMD        // a command
// T_OPER       // an operator
// T_FUN        // a function (also used for a function that can operate as a command)
// T_FNA        // a function that has no arguments
//

var
  FilesList: TStringList;

type tagset=set of (cmd, fP, fN, op, NT, MX170, MX470);
// extra tag items added for MX170 and MX470 specific items

var T:text;
    I,B,C,D:integer;
    tag:tagset;
    ch:char;
    matched:boolean;
    S,filename:string;

var tokenlist:array [0..499] of record
                                  name:string;
                                  tag:tagset
                                end;
    TC:integer;

begin
  FilesList:=TStringList.Create;

// build command and function table
  try
    FindFiles(FilesList,'..\Source','*.h');

    writeln('########## searching header files ##########,',stringofchar('#',16),',##,##,##,##,##,##,##');

    TC:=0;

    for I:=0 to FilesList.Count-1 do
    begin
      filename:=FilesList.Strings[I];
      assignfile(T,filename);

      reset(T);
      matched:=false;
      while not eof(T) do
      begin
        S:='';
        repeat
          read(T,ch);
          if not (ch in [#10,#13]) then S:=S+ch;
        until eof(T) or (ch in [#10,#13]);

        B:=pos('{',S);
        C:=pos('}',s);
        D:=pos('T_INV',S)+pos('T_NA',S)+
           pos('T_CMD',S)+pos('T_OPER',S)+
           pos('T_FUN',S)+pos('T_FNA',S);

        tag:=[];
        if pos('MX170',filename)<>0 then tag:=[MX170];
        if pos('MX470',filename)<>0 then tag:=[MX470];

        if pos('T_CMD',S)<>0 then tag:=tag+[cmd];
        if (pos('T_FUN',S)<>0) then tag:=tag+[fP];
        if pos('T_FNA',S)<>0 then tag:=tag+[fN];
        if pos('T_OPER',S)<>0 then tag:=tag+[op];
        if pos('T_NA',S)<>0 then tag:=tag+[NT];

        if ((B*C)<>0) and (B<C) and (D<>0) then
        begin
          if not matched then
          begin
            write(FilesList.Strings[I]);
            matched:=true
          end;

          B:=pos('"',S);
          C:=B;
          if B<>0 then C:=pos('"',copy(S,B+1,maxint));
          if C<>0 then begin
                         write(',',copy(S,B+1,C-1),stringofchar(' ',16-C),',');
                         if cmd in tag then write('C,')
                                       else write(' ,');
                         if fP in tag then write('fP,')
                                      else write('  ,');
                         if fN in tag then write('fN,')
                                      else write('  ,');
                         if op in tag then write('O,')
                                      else write(' ,');
                         if NT in tag then write('NT,')
                                      else write('  ,');
                         if MX170 in tag then write('1,')
                                         else write(' ,');
                         if MX470 in tag then writeln('4')
                                         else writeln(' ');

                         tokenlist[TC].name:=copy(S,B+1,C-1);
                         tokenlist[TC].tag:=tag;
                         inc(TC)
                       end
                  else writeln('FORMAT ERROR: ',S)
        end
      end;
      if matched then writeln;
      close(T)
    end
  finally
    FilesList.Free
  end;

  writeln;
  writeln;

// sort command and function list
  for B:=0 to TC-1 do
  for C:=B to TC-1 do
  if tokenlist[B].name>tokenlist[C].name then
  begin
    S:=tokenlist[B].name;
    tokenlist[B].name:=tokenlist[C].name;
    tokenlist[C].name:=S;
    tag:=tokenlist[B].tag;
    tokenlist[B].tag:=tokenlist[C].tag;
    tokenlist[C].tag:=tag
  end;

// output sorted list
  writeln('########## sorted list of tokens ###########');

  for B:=0 to TC-1 do
  begin
    write(',',tokenlist[B].name,',');
    tag:=tokenlist[B].tag;
    if cmd in tag then write('C,')
                  else write(' ,');
    if fP in tag then write('fP,')
                 else write('  ,');
    if fN in tag then write('fN,')
                 else write('  ,');
    if op in tag then write('O,')
                 else write(' ,');
    if NT in tag then write('NT,')
                 else write('  ,');
    if MX170 in tag then write('1,')
                    else write(' ,');
    if MX470 in tag then writeln('4')
                    else writeln(' ')
  end;

  writeln;
  writeln;
  writeln;

  //
  FilesList:=TStringList.Create;
  try
    FindFiles(FilesList,'..\Source','*.c');

    writeln('########## searching C files ###############');

    for I:=0 to FilesList.Count-1 do
    begin
      assignfile(T,FilesList.Strings[I]);
      reset(T);
      matched:=false;
      while not eof(T) do
      begin
        S:='';
        repeat
          read(T,ch);
          if not (ch in [#10,#13]) then S:=S+ch;
        until eof(T) or (ch in [#10,#13]);

        D:=pos('checkstring',S);
        B:=pos('"',S);

        if (D<>0) and (B>D)then
        begin
          if not matched then
          begin
            writeln('extracted from ',FilesList.Strings[I]);
            matched:=true
          end;

          B:=pos('"',S);
          C:=B;
          if B<>0 then C:=pos('"',copy(S,B+1,maxint));
          if C<>0 then writeln(' ,',copy(S,B+1,C-1))
                  else writeln(S)
        end
      end;
      if matched then writeln;
      close(T)
    end
  finally
    FilesList.Free
  end
end.
